@@ -12,7 +12,7 @@ unset-dev-version () {
 export OCAMLRUNPARAM=b
 
 (set +x ; echo -en "::group::build opam\r") 2>/dev/null
-if [[ $OPAM_TEST -eq 1 ]] ; then
+if [[ "$OPAM_TEST" -eq 1 ]] || [[ "$OPAM_DOC" -eq 1 ]] ; then
   export OPAMROOT=$OPAMBSROOT
   # If the cached root is newer, regenerate a binary compatible root
   opam env || { rm -rf $OPAMBSROOT; init-bootstrap; }
@@ -38,7 +38,7 @@ if [ $OPAM_UPGRADE -eq 1 ]; then
 fi
 # Disable implicit transitive deps
 sed -i -e '/(implicit_transitive_deps /s/true/false/' dune-project
-make all admin
+make all admin opam-stripped
 sed -i -e '/(implicit_transitive_deps /s/false/true/' dune-project
 
 rm -f "$PREFIX/bin/opam"
@@ -48,6 +48,49 @@ make install
 export PATH="$PREFIX/bin:$PATH"
 opam --version
 
+if [[ "$OPAM_DOC" -eq 1 ]]; then
+  make -C doc html man-html pages
+
+  if [ "$GITHUB_EVENT_NAME" = "pull_request" ]; then
+    . .github/scripts/common/hygiene-preamble.sh
+    diff="git diff $BASE_REF_SHA..$PR_REF_SHA"
+    set +e
+    files=$($diff --name-only --diff-filter=A -- src/**/*.mli)
+    set -e
+    if [ -n "$files" ]; then
+      echo '::group::new module(s) added - check index updated too'
+      if $diff --name-only --exit-code -- doc/index.html ; then
+        echo '::error new module(s) added but doc/index.html not updated'
+        echo "$files"
+        exit 3
+      fi
+      echo '::endgroup::new module added - checking it'
+    else
+      echo 'No new modules added'
+    fi
+  fi
+
+  mapfile -t htmlfiles < <(ls doc/pages/*.md | sed -e 's/\.md$/.html/')
+  mapfile -t manfiles < <(opam help topics | sed -e 's|.*|doc/man-html/opam-&.html|')
+  mapfile -O "${#manfiles[@]}" -t manfiles < <(opam admin help topics | sed -e 's|.*|doc/man-html/opam-admin-&.html|')
+
+  echo '::group::checking for generated files'
+  echo "pages: $htmlfiles"
+  echo "topics: $manfiles"
+  files=("${htmlfiles[@]}" "${manfiles[@]}")
+  missing=""
+  for file in "${files[@]}"; do
+    if ! test -f "$file" ; then
+      missing="$missing '$file'"
+    fi
+  done
+  if [ "x$missing" != "x" ]; then
+    echo "::error missing generated doc files: $missing"
+    exit 4
+  fi
+  echo '::endgroup::checking generated files'
+fi
+
 if [ "$OPAM_TEST" = "1" ]; then
   # test if an upgrade is needed
   set +e
@@ -84,16 +127,21 @@ if [ "$OPAM_TEST" = "1" ]; then
   fi
   cd $CACHE/opam-rt
   git fetch origin
-  if git ls-remote --exit-code origin $BRANCH ; then
-    if git branch | grep -q $BRANCH; then
-      git checkout $BRANCH
-      git reset --hard origin/$BRANCH
-    else
-      git checkout -b $BRANCH origin/$BRANCH
-    fi
+  if [ "$GITHUB_EVENT_NAME" = "pull_request" ] && git ls-remote --exit-code origin "$GITHUB_PR_USER/$BRANCH" ; then
+    BRANCH=$GITHUB_PR_USER/$BRANCH
+  fi
+  if git ls-remote --exit-code origin "$BRANCH"; then
+    OPAM_RT_BRANCH=$BRANCH
+  elif [ "$GITHUB_EVENT_NAME" = pull_request ] && git ls-remote --exit-code origin "$GITHUB_BASE_REF"; then
+    OPAM_RT_BRANCH=$GITHUB_BASE_REF
   else
-    git checkout master
-    git reset --hard origin/master
+    OPAM_RT_BRANCH=master
+  fi
+  if git branch | grep -q "$OPAM_RT_BRANCH"; then
+    git checkout "$OPAM_RT_BRANCH"
+    git reset --hard "origin/$OPAM_RT_BRANCH"
+  else
+    git checkout -b "$OPAM_RT_BRANCH" "origin/$OPAM_RT_BRANCH"
   fi
 
   test -d _opam || opam switch create . --no-install --formula '"ocaml-system"'
