@@ -99,7 +99,7 @@ let index_command cli =
         let module VCS = (val OpamRepository.find_backend_by_kind vcs) in
         match OpamProcess.Job.run (VCS.revision repo_root) with
         | None -> date ()
-        | Some hash -> OpamPackage.Version.to_string hash
+        | Some hash -> hash
     in
     let repo_def = OpamFile.Repo.with_stamp repo_stamp repo_def in
     OpamFile.Repo.write repo_file repo_def;
@@ -343,13 +343,10 @@ let update_extrafiles_command cli =
         field as requested."
   ]
   in
-  let hash_kinds = [`MD5; `SHA256; `SHA512] in
   let hash_type_arg =
     OpamArg.mk_opt ~cli OpamArg.(cli_from cli2_2) ["hash"]
       "HASH_ALGO" "The hash, or hashes to be added"
-      Arg.(some (enum
-                   (List.map (fun k -> OpamHash.string_of_kind k, k)
-                      hash_kinds))) None
+      (Arg.some OpamArg.hash_kinds) None
   in
   let packages =
     OpamArg.mk_opt ~cli OpamArg.(cli_from cli2_2) ["p";"packages"]
@@ -425,6 +422,121 @@ let update_extrafiles_command cli =
     Term.(const cmd $ global_options cli
           $ hash_type_arg $ packages)
 
+let migrate_extrafiles_command_doc =
+  "Move extra-files to extra-source."
+let migrate_extrafiles_command cli =
+  let command = "migrate-extrafiles" in
+  let doc = migrate_extrafiles_command_doc in
+  let man = [
+    `S Manpage.s_description;
+    `P "This command scans through package definitions, and migrates all \
+        $(i,extra-files) fields to $(i,extra-source) as requested."
+  ] in
+  let hash_type_arg =
+    OpamArg.mk_opt ~cli OpamArg.(cli_from cli2_4) ["hash"]
+      "HASH_ALGO" "The hash, or hashes to be added"
+      OpamArg.hash_kinds `SHA256
+  in
+  let packages =
+    OpamArg.mk_opt ~cli OpamArg.(cli_from cli2_4) ["p";"packages"]
+      "PACKAGES" "Only add extra files for the given packages"
+      Arg.(list OpamArg.package) []
+  in
+  let local_dir_arg =
+    let doc =
+      Arg.info ~docv:"DIR"
+        ~doc:"Name of the local directory where to put the extra-files. \
+              They will be put into \
+              DIR/patches/<pkgname>/<pkgname.version>/filenameYY" []
+    in
+    Arg.(required & pos 0 (some OpamArg.dirname) None & doc)
+  and url_prefix_arg =
+    let doc =
+      Arg.info ~docv:"URL"
+        ~doc:"Prefix of the URL to emit into extra-sources." []
+    in
+    Arg.(required & pos 1 (some OpamArg.url) None & doc)
+  in
+  let cmd global_options kind packages local_dir url_prefix () =
+    OpamArg.apply_global_options cli global_options;
+    let repo_root = checked_repo_root () in
+    let pkg_prefixes = packages_with_prefixes repo_root packages in
+    let compute ?kind file =
+      OpamHash.compute ?kind (OpamFilename.to_string file)
+    in
+    let has_error =
+      OpamPackage.Map.fold (fun nv prefix has_error ->
+          let opam_file = OpamRepositoryPath.opam repo_root prefix nv in
+          let opam = OpamFile.OPAM.read opam_file in
+          let has_error =
+            let urlfile = OpamRepositoryPath.url repo_root prefix nv in
+            if OpamFile.exists urlfile then
+              (OpamConsole.warning
+                 "Not updating external URL file at %s, \
+                  consider upgrading your repository with 'opam admin upgrade'"
+                 (OpamFile.to_string urlfile);
+               true)
+            else has_error
+          in
+          let files_dir = OpamRepositoryPath.files repo_root prefix nv in
+          if OpamFilename.exists_dir files_dir then
+            (let files =
+               OpamFilename.rec_files files_dir
+               |> List.map (fun file ->
+                   file,
+                   OpamFilename.Base.of_string
+                     (OpamFilename.remove_prefix files_dir file))
+             in
+             match files with
+             | [] -> has_error
+             | _ ->
+               let hashes =
+                 List.map (fun (file, base) ->
+                     let xhash = compute ~kind file in
+                     file, base, xhash)
+                   files
+               in
+               let patch_prefix = "patches" in
+               let extra_sources =
+                 List.map (fun (src, base, hash) ->
+                     let name = OpamPackage.name_to_string nv in
+                     let name_version = OpamPackage.to_string nv in
+                     let dst_dir = OpamFilename.Op.(
+                         local_dir / patch_prefix / name / name_version
+                       ) in
+                     let dst = OpamFilename.create dst_dir base in
+                     (* copy file to dst, remove from files *)
+                     OpamFilename.mkdir dst_dir;
+                     OpamFilename.copy ~src ~dst;
+                     let url =
+                       let file = OpamFilename.Base.to_string base in
+                       OpamFile.URL.create ~checksum:[hash]
+                         OpamUrl.Op.(url_prefix / patch_prefix
+                                     / name / name_version/ file)
+                     in
+                     base, url)
+                   hashes
+               in
+               OpamFilename.cleandir files_dir;
+               OpamFilename.rmdir_cleanup files_dir;
+               let my_extra_sources = OpamFile.OPAM.extra_sources opam in
+               let opam1 =
+                 OpamFile.OPAM.with_extra_sources
+                   (my_extra_sources @ extra_sources) opam
+               in
+               let opam1 = OpamFile.OPAM.with_extra_files_opt None opam1 in
+               OpamFile.OPAM.write_with_preserved_format opam_file opam1;
+               has_error)
+          else
+            has_error)
+        pkg_prefixes false
+    in
+    if has_error then OpamStd.Sys.exit_because `Sync_error
+    else OpamStd.Sys.exit_because `Success
+  in
+  OpamArg.mk_command ~cli OpamArg.(cli_from cli2_4) command ~doc ~man
+    Term.(const cmd $ global_options cli $ hash_type_arg $ packages
+          $ local_dir_arg $ url_prefix_arg)
 
 let add_hashes_command_doc =
   "Add archive hashes to an opam repository."
@@ -438,12 +550,9 @@ let add_hashes_command cli =
         in <opamroot>/download-cache/hash-cache for subsequent runs.";
   ]
   in
-  let hash_kinds = [`MD5; `SHA256; `SHA512] in
   let hash_types_arg =
     OpamArg.nonempty_arg_list "HASH_ALGO" "The hash, or hashes to be added"
-      (Arg.enum
-         (List.map (fun k -> OpamHash.string_of_kind k, k)
-            hash_kinds))
+      OpamArg.hash_kinds
   in
   let packages =
     OpamArg.mk_opt ~cli OpamArg.(cli_from cli2_1) ["p";"packages"]
@@ -455,7 +564,7 @@ let add_hashes_command cli =
       "Replace the existing hashes rather than adding to them"
   in
   let create_hash_tables hash_cache_dir =
-    let t = Hashtbl.create (List.length hash_kinds) in
+    let t = Hashtbl.create (List.length OpamHash.all_kinds) in
     List.iter (fun k1 ->
         List.iter (fun k2 ->
             if k1 <> k2 then (
@@ -474,9 +583,9 @@ let add_hashes_command cli =
                | _ -> failwith ("Bad cache at "^OpamFile.to_string cache_file));
               Hashtbl.add t (k1,k2) (cache_file, t_mapping);
             ))
-          hash_kinds
+          OpamHash.all_kinds
       )
-      hash_kinds;
+      OpamHash.all_kinds;
     t
   in
   let save_hashes hash_tables =
@@ -761,11 +870,6 @@ let check_command cli =
         are supposed to be available. By default, all checks are run."
   ]
   in
-  let ignore_test_arg =
-    OpamArg.mk_flag ~cli OpamArg.cli_original ["ignore-test-doc";"i"]
-      "By default, $(b,{with-test}) and $(b,{with-doc}) dependencies are \
-       included. This ignores them, and makes the test more tolerant."
-  in
   let print_short_arg =
     OpamArg.mk_flag ~cli OpamArg.cli_original ["s";"short"]
       "Only output a list of uninstallable packages"
@@ -782,7 +886,7 @@ let check_command cli =
     OpamArg.mk_flag ~cli OpamArg.cli_original ["obsolete"]
       "Analyse for obsolete packages"
   in
-  let cmd global_options ignore_test print_short
+  let cmd global_options print_short
       installability cycles obsolete () =
     OpamArg.apply_global_options cli global_options;
     let repo_root = checked_repo_root () in
@@ -793,7 +897,7 @@ let check_command cli =
     in
     let pkgs, unav_roots, uninstallable, cycle_packages, obsolete =
       OpamAdminCheck.check
-        ~quiet:print_short ~installability ~cycles ~obsolete ~ignore_test
+        ~quiet:print_short ~installability ~cycles ~obsolete
         repo_root
     in
     let all_ok =
@@ -827,9 +931,74 @@ let check_command cli =
     OpamStd.Sys.exit_because (if all_ok then `Success else `False)
   in
   OpamArg.mk_command  ~cli OpamArg.cli_original command ~doc ~man
-  Term.(const cmd $ global_options cli $ ignore_test_arg $ print_short_arg
+  Term.(const cmd $ global_options cli $ print_short_arg
         $ installability_arg $ cycles_arg $ obsolete_arg)
 
+let compare_versions_command_doc = "Compare two package versions"
+let compare_versions_command cli =
+  let operator : OpamFormula.relop option Term.t =
+     let make_flag (relop : OpamFormula.relop) =
+       let flag_name =
+         match relop with
+         | `Eq -> "eq"
+         | `Neq -> "neq"
+         | `Geq -> "geq"
+         | `Gt -> "gt"
+         | `Leq -> "leq"
+         | `Lt -> "lt"
+       in
+       let doc = Printf.sprintf "assert V0 %s V1" (OpamFormula.string_of_relop relop) in
+       OpamArg.(cli_from cli2_4), Some relop, [ flag_name ], doc
+     in
+     OpamArg.mk_vflag ~cli None (List.map make_flag OpamFormula.all_relop)
+  in
+  let version_arg n =
+    let doc =
+      Arg.info
+        ~docv:(Printf.sprintf "V%d" n)
+        ~doc:"Package version to compare" []
+    in
+    Arg.(required & pos n (some OpamArg.package_version) None & doc)
+  in
+  let command = "compare-versions" in
+  let doc = compare_versions_command_doc in
+  let man = [
+    `S Manpage.s_description;
+    `P "This command compares 2 package versions. By default it outputs 'V0 OP \
+        V1' to the console with OP in {<,>,=} such that the equation holds. \
+        When an operator is supplied, the output is suppressed and the result \
+        of the comparison is checked against the provided operator: the command \
+        exits 0 if the comparison holds, and 1 otherwise. For example:";
+    `Pre "\n\
+          \\$ opam admin compare-versions 0.0.9 0.0.10\n\
+          0.0.9 < 0.0.10\n\
+          \n\
+          \\$ opam admin compare-versions 0.0.9 --lt 0.0.10\n\
+          [0]\n\
+          \n\
+          \\$ opam admin compare-versions 0.0.9 --eq 0.0.10\n\
+          [1]";
+    `S Manpage.s_arguments;
+    `S Manpage.s_options;
+  ]
+  in
+  let cmd v0 v1 operator () =
+    match operator with
+    | None ->
+      let result = OpamPackage.Version.compare v0 v1 in
+      OpamConsole.formatted_msg "%s %s %s\n"
+        (OpamPackage.Version.to_string v0)
+        (if result < 0 then "<" else if result = 0 then "=" else ">")
+        (OpamPackage.Version.to_string v1)
+    | Some operator ->
+      OpamStd.Sys.exit_because
+        (if OpamFormula.eval_relop operator v0 v1
+         then `Success
+         else `False)
+  in
+  OpamArg.mk_command ~cli OpamArg.(cli_from cli2_4) command ~doc ~man
+    Term.(const cmd $ version_arg 0 $ version_arg 1 $ operator)
+
 let pattern_list_arg =
   OpamArg.arg_list "PATTERNS"
     "Package patterns with globs. matching against $(b,NAME) or \
@@ -1227,11 +1396,13 @@ let admin_subcommands cli =
     upgrade_command cli;
     lint_command cli;
     check_command cli;
+    compare_versions_command cli;
     list_command cli;
     filter_command cli;
     add_constraint_command cli;
     add_hashes_command cli;
     update_extrafiles_command cli;
+    migrate_extrafiles_command cli;
     help;
   ]
 
