@@ -186,6 +186,16 @@ let environment_variables =
          "change the time allowance of the solver. Default is %.1f, set to 0 \
           for unlimited. Note that all solvers may not support this option."
          (OpamStd.Option.default 0. OpamSolverConfig.(default.solver_timeout)));
+      "SOLVERTOLERANCE", cli_from cli2_4, (fun v -> SOLVERTOLERANCE (env_float v)),
+      (Printf.sprintf
+         "changes the tolerance towards the solver choosing an unoptimized \
+          solution (i.e. might pull outdated packages). Typical values range \
+          from 0.0 (best solution known to the solver) to 1.0 (unoptimized \
+          solution). Default is %.1f. This option is useful in case the solver \
+          can't find a solution in a reasonable time \
+          (see $(b,\\$OPAMSOLVERTIMEOUT)). Note that all solvers may not \
+          support this option."
+         (OpamStd.Option.default 0. OpamSolverConfig.default.solver_tolerance));
       "UPGRADECRITERIA", cli_original,
       (fun v -> UPGRADECRITERIA (env_string v)),
       "specifies user $(i,preferences) for dependency solving when performing \
@@ -302,7 +312,7 @@ let environment_variables =
       "see option `--show'.";
       "SKIPUPDATE", cli_original, (fun v -> SKIPUPDATE (env_bool v)),
       "see option `--skip-updates'.";
-      "STATS", cli_original, (fun v -> STATS (env_bool v)),
+      "STATS", cli_between cli2_0 cli2_4, (fun _v -> OpamStd.Config.E.REMOVED),
       "display stats at the end of command.";
       "VERBOSEON", cli_from cli2_2, (fun v -> VERBOSEON (env_string_list v)),
       "see option --verbose-on";
@@ -565,7 +575,6 @@ let apply_global_options cli o =
     (* - client options - *)
     ?working_dir:(flag o.working_dir)
     ?ignore_pin_depends:(flag o.ignore_pin_depends)
-    (* ?print_stats:bool *)
     (* ?sync_archives:bool *)
     (* ?pin_kind_auto:bool *)
     (* ?autoremove:bool *)
@@ -1058,6 +1067,10 @@ let opamlist_columns =
   in
   parse, print
 
+let hash_kinds =
+  Arg.enum
+    (List.map (fun k -> OpamHash.string_of_kind k, k) OpamHash.all_kinds)
+
 let mk_flag ~cli validity ?(section=Manpage.s_options) flags name =
   mk_flag ~cli validity ~section flags name
 
@@ -1412,6 +1425,17 @@ let lock_suffix ?section cli =
     "Set locked files suffix to $(i,SUFFIX)."
     Arg.(string) ("locked")
 
+(* Checksums options *)
+let no_checksums ?section cli from_cli =
+  mk_flag ~cli from_cli ?section ["no-checksums"]
+    "Do not verify the checksum of downloaded archives.\
+     This is equivalent to setting $(b,\\$OPAMNOCHECKSUMS) to \"true\"."
+let require_checksums ?section cli from_cli =
+  mk_flag ~cli from_cli ?section ["require-checksums"]
+    "Reject the installation of packages that don't provide a checksum for\
+     the upstream archives.  This is equivalent to setting \
+     $(b,\\$OPAMREQUIRECHECKSUMS) to \"true\"."
+
 (* Options common to all build commands *)
 let build_option_section = "PACKAGE BUILD OPTIONS"
 let man_build_option_section =
@@ -1440,14 +1464,8 @@ let build_options cli =
        affects packages that are explicitly listed on the command-line. \
        This is equivalent to setting $(b,\\$OPAMINPLACEBUILD) to \"true\"."
   in
-  let no_checksums =
-    mk_flag ~cli cli_original ~section ["no-checksums"]
-      "Do not verify the checksum of downloaded archives.\
-       This is equivalent to setting $(b,\\$OPAMNOCHECKSUMS) to \"true\"." in
-  let req_checksums =
-    mk_flag ~cli cli_original ~section ["require-checksums"]
-      "Reject the installation of packages that don't provide a checksum for the upstream archives. \
-       This is equivalent to setting $(b,\\$OPAMREQUIRECHECKSUMS) to \"true\"." in
+  let no_checksums = no_checksums ~section cli cli_original in
+  let req_checksums = require_checksums ~section cli cli_original in
   let build_test =
     mk_flag_replaced ~cli ~section [
       cli_between cli2_0 cli2_1 ~replaced:"--with-test", ["build-test"];
