@@ -25,6 +25,7 @@ val cli2_0: OpamCLIVersion.t
 val cli2_1: OpamCLIVersion.t
 val cli2_2: OpamCLIVersion.t
 val cli2_3: OpamCLIVersion.t
+val cli2_4: OpamCLIVersion.t
 
 (* [cli_from ?platform ?experimental since] validity flag since [since], and no
    removal version. If [experimental] is true, it is marked as is (warning and
@@ -90,7 +91,7 @@ val mk_vflag_all:
   (validity * 'a * string list * string) list ->
   'a list Term.t
 
-(* Escaped Windows directory separator. To use instead of [Filename.dir_sep] for
+(* Escaped Windows directory separator. To use instead of {!Filename.dir_sep} for
    manpage strings *)
 val dir_sep: string
 
@@ -212,6 +213,12 @@ val apply_build_options: OpamCLIVersion.Sourced.t -> build_options -> unit
 val locked: ?section:string -> OpamCLIVersion.Sourced.t -> bool Term.t
 val lock_suffix: ?section:string -> OpamCLIVersion.Sourced.t -> string Term.t
 
+(** Checksum options *)
+val no_checksums:
+  ?section:string -> OpamCLIVersion.Sourced.t -> validity -> bool Term.t
+val require_checksums:
+  ?section:string -> OpamCLIVersion.Sourced.t -> validity -> bool Term.t
+
 (** {3 Package listing and filtering options} *)
 
 (** Man section name *)
@@ -292,6 +299,8 @@ val warn_selector: (int * [`Enable | `Disable | `EnableError]) list Arg.conv
 
 val opamlist_columns: OpamListCommand.output_format list Arg.conv
 
+val hash_kinds : OpamHash.kind Arg.conv
+
 (** {2 Subcommands} *)
 
 type 'a subcommand = validity * string * 'a * string list * string
@@ -318,7 +327,7 @@ val enum_with_default: (string * 'a default) list -> 'a Arg.converter
 val mk_subcommands_with_default:
   cli:OpamCLIVersion.Sourced.t ->
   'a default subcommands -> 'a option Term.t * string list Term.t
-(** Same as {!mk_subcommand} but use the default value if no
+(** Same as {!mk_subcommands} but use the default value if no
     sub-command is selected. *)
 
 val bad_subcommand:
