@@ -64,14 +64,18 @@ let remove_files_from_destdir st pfx packages =
            (OpamConsole.colorise `bold (OpamFilename.to_string f));
        if not OpamStateConfig.(!r.dryrun) then OpamFilename.remove f)
     else if OpamFilename.exists_dir d then
-      if OpamFilename.dir_is_empty d then
-        (if OpamConsole.verbose () then
-           OpamConsole.msg "Removing %s\n"
-             (OpamConsole.colorise `bold (OpamFilename.Dir.to_string d));
-         if not OpamStateConfig.(!r.dryrun) then OpamFilename.rmdir d)
-      else
+      begin match OpamFilename.dir_is_empty d with
+      | Some true ->
+        if OpamConsole.verbose () then
+          OpamConsole.msg "Removing %s\n"
+            (OpamConsole.colorise `bold (OpamFilename.Dir.to_string d));
+        if not OpamStateConfig.(!r.dryrun) then
+          OpamFilename.rmdir d
+      | Some false ->
         OpamConsole.note "Not removing non-empty directory %s"
           (OpamConsole.colorise `bold (OpamFilename.Dir.to_string d))
+      | None -> ()
+      end
   | _ -> ()
 
 let name_from_project_dirname d =
@@ -250,7 +254,7 @@ let resolve_locals ?(quiet=false) ?locked ?recurse ?subpath
              (OpamUrl.to_string nf.pin.pin_url))
           duplicates)
 
-let autopin_aux st ?quiet ?(for_view=false) ?recurse ?subpath ?locked
+let autopin_aux st ?quiet ?recurse ?subpath ?locked
     atom_or_local_list =
   let to_pin, atoms =
     resolve_locals ?quiet ?recurse ?subpath ?locked atom_or_local_list
@@ -293,27 +297,39 @@ let autopin_aux st ?quiet ?(for_view=false) ?recurse ?subpath ?locked
           (* check of the target to avoid repin of pin to update with `opam
              install .` and loose edited opams *)
           let pinned_pkg = OpamPinned.package st nf.pin_name in
-          OpamSwitchState.primary_url st pinned_pkg = Some nf.pin.pin_url
-          &&
-          (match OpamSwitchState.opam_opt st pinned_pkg with
-           | Some opam ->
-             (match locked, OpamFile.OPAM.locked opam with
-              | Some ext , Some ext' -> String.equal ext ext'
-              | None, None -> true
-              | _ -> false)
-           | None -> false)
-          &&
-          (* For `opam show`, we need to check does the opam file changed to
-             perform a simulated pin if so *)
-          (not for_view ||
-           match
-             OpamSwitchState.opam_opt st pinned_pkg,
-             OpamFile.OPAM.read_opt nf.pin.pin_file
-           with
-           | Some opam0, Some opam ->
-             let opam = OpamFile.OPAM.with_locked_opt nf.pin.pin_locked opam in
-             OpamFile.OPAM.equal opam0 opam
-           | _, _ -> false)
+          match OpamSwitchState.opam_opt st pinned_pkg with
+          | Some opam0 ->
+            (match OpamFile.OPAM.read_opt nf.pin.pin_file with
+             | Some opam ->
+               (* Keep synchronised with [OpamFile.OPAM.pp_raw_fields] added
+                  fields. *)
+               let opam = OpamFile.OPAM.with_locked_opt nf.pin.pin_locked opam in
+               let opam =
+                 match OpamFile.OPAM.name_opt opam with
+                 | None -> OpamFile.OPAM.with_name nf.pin_name opam
+                 | Some _ -> opam
+               in
+               let opam =
+                 match OpamFile.OPAM.version_opt opam with
+                 | None ->
+                   OpamFile.OPAM.with_version
+                     (OpamPinCommand.default_version st nf.pin_name) opam
+                 | Some _ -> opam
+               in
+               let opam =
+                 match OpamFile.OPAM.url opam with
+                 | None ->
+                   OpamFile.OPAM.with_url
+                     (OpamFile.URL.create ?subpath:nf.pin.pin_subpath
+                        nf.pin.pin_url)
+                     opam
+                 | Some _ -> opam
+               in
+               OpamStd.Option.equal String.equal
+                 locked (OpamFile.OPAM.locked opam)
+               && OpamFile.OPAM.effectively_equal opam0 opam
+             | None -> false)
+          | None -> false
         with Not_found -> false)
       to_pin
   in
@@ -374,6 +390,15 @@ let simulate_local_pinnings ?quiet ?(for_view=false) st to_pin =
       ++ local_packages
     else st.pinned
   in
+  let overwrote_opams =
+    if for_view then OpamPackage.Map.empty else
+      OpamPackage.Map.filter_map (fun nv opam ->
+          if OpamPackage.Set.mem nv local_packages then
+             Some (OpamPackage.Set.mem nv st.pinned, opam)
+           else
+             None)
+        st.opams
+  in
   let st = {
     st with
     opams =
@@ -389,14 +414,39 @@ let simulate_local_pinnings ?quiet ?(for_view=false) st to_pin =
            st.switch_global st.switch st.switch_config ~pinned
            ~opams:local_opams)
     );
+    reinstall = lazy (
+      let open OpamPackage.Set.Op in
+      let installed_pinned = local_packages %% st.installed in
+      OpamPackage.Set.fold (fun pkg reinstall ->
+          let old_opam = OpamPackage.Map.find pkg st.installed_opams in
+          let new_opam = OpamPackage.Map.find pkg local_opams in
+          if OpamFile.OPAM.effectively_equal old_opam new_opam then
+            reinstall
+          else
+            OpamPackage.Set.add pkg reinstall)
+        installed_pinned (Lazy.force st.reinstall)
+    );
     pinned;
+    overwrote_opams;
   } in
   st, local_packages
 
+let simulate_pinned_atoms pins atoms =
+  (* Simulate pinning functions return atoms with no versions constraint, we
+     only update those to ensure that this pinned version is taken as the
+     switch state is not updated. *)
+  List.map (function
+      | (_name, Some _) as a -> a
+      | (name, None) as a ->
+        match OpamPackage.package_of_name_opt pins name with
+        | Some pkg -> (name, Some (`Eq, OpamPackage.version pkg))
+        | None -> a)
+    atoms
+
 let simulate_autopin st ?quiet ?(for_view=false) ?locked ?recurse ?subpath
     atom_or_local_list =
   let atoms, to_pin, obsolete_pins, already_pinned_set =
-    autopin_aux st ?quiet ~for_view ?recurse ?subpath ?locked atom_or_local_list
+    autopin_aux st ?quiet ?recurse ?subpath ?locked atom_or_local_list
   in
   if to_pin = [] then st, atoms else
   let st =
@@ -424,7 +474,7 @@ let simulate_autopin st ?quiet ?(for_view=false) ?locked ?recurse ?subpath
         OpamConsole.note "The following may not reflect the above pinnings (their \
                           package definitions are not available at this stage)";
         OpamConsole.msg "\n"));
-  st, atoms
+  st, simulate_pinned_atoms pins atoms
 
 let autopin st ?(simulate=false) ?quiet ?locked ?recurse ?subpath
     atom_or_local_list =
@@ -485,6 +535,7 @@ let autopin st ?(simulate=false) ?quiet ?locked ?recurse ?subpath
       OpamStd.Sys.exit_because `Aborted
   in
   let _result, st, _updated =
+    if simulate then false, st, OpamPackage.Set.empty else
     let already_pinned =
       OpamPackage.Set.union already_pinned_set
         (OpamPackage.packages_of_names pins already_pinned_diff_url)
@@ -495,12 +546,14 @@ let autopin st ?(simulate=false) ?quiet ?locked ?recurse ?subpath
       OpamUpdate.dev_packages st ~working_dir:OpamPackage.Set.empty already_pinned
   in
   let st =
-    if OpamClientConfig.(!r.ignore_pin_depends) then st else
+    (* NOTE: We skip pin-depends on non-simulated because handle_pin_depends has
+       already been called in source_pin *)
+    if not simulate || OpamClientConfig.(!r.ignore_pin_depends) then st else
       OpamPackage.Set.fold (fun nv st ->
           OpamPinCommand.handle_pin_depends st nv (OpamSwitchState.opam st nv))
         (OpamPackage.Set.union pins already_pinned_set) st
   in
-  st, atoms
+  st, simulate_pinned_atoms pins atoms
 
 let check_and_revert_sandboxing root config =
   let sdbx_wrappers =
