@@ -491,12 +491,10 @@ and source_pin
       version, None
   in
 
-  if not (OpamPackage.has_name st.packages name) &&
-     not (OpamConsole.confirm
-            "Package %s does not exist, create as a %s package?"
-            (OpamPackage.Name.to_string name)
-            (OpamConsole.colorise `bold "NEW"))
-  then raise Aborted;
+  if not (OpamPackage.has_name st.packages name) then
+    OpamConsole.note "Package %s does not exist in opam repositories \
+                      registered in the current switch."
+      (OpamPackage.Name.to_string name);
 
   (match OpamStd.Option.map OpamFile.URL.url cur_urlf, target_url with
    | Some u, Some target when OpamUrl.(
@@ -571,37 +569,34 @@ and source_pin
     | _ -> opam_opt
   in
 
-  if not need_edit && opam_opt = None then
-    OpamConsole.note
-      "No package definition found for %s: please complete the template"
-      (OpamConsole.colorise `bold (OpamPackage.to_string nv));
-
-  let need_edit = need_edit || opam_opt = None in
-
   let opam_opt =
-    let opam_base = match opam_opt with
-      | None -> OpamFileTools.template nv
-      | Some opam -> opam
-    in
-    let opam_base =
-      OpamFile.OPAM.with_url_opt urlf opam_base
-    in
     if need_edit then
-      (if not (OpamFile.exists temp_file) then
-         OpamFile.OPAM.write_with_preserved_format
-           ?format_from:(OpamPinned.orig_opam_file st name opam_base)
-           temp_file opam_base;
-       edit_raw name temp_file >>|
-       (* Preserve metadata_dir so that copy_files below works *)
-       OpamFile.OPAM.(with_metadata_dir (metadata_dir opam_base))
-      )
+      let opam_base = match opam_opt with
+        | None ->
+          OpamConsole.note
+            "No package definition found for %s: please complete the template"
+            (OpamConsole.colorise `bold (OpamPackage.to_string nv));
+          OpamFileTools.template nv
+        | Some opam -> opam
+      in
+      let opam_base =
+        OpamFile.OPAM.with_url_opt urlf opam_base
+      in
+      if not (OpamFile.exists temp_file) then
+        OpamFile.OPAM.write_with_preserved_format
+          ?format_from:(OpamPinned.orig_opam_file st name opam_base)
+          temp_file opam_base;
+      edit_raw name temp_file >>|
+      (* Preserve metadata_dir so that copy_files below works *)
+      OpamFile.OPAM.(with_metadata_dir (metadata_dir opam_base))
     else
-      Some opam_base
+      Option.map (OpamFile.OPAM.with_url_opt urlf) opam_opt
   in
   match opam_opt with
   | None ->
     OpamConsole.error_and_exit `Not_found
-      "No valid package definition found"
+      "No valid package definition found for %s"
+      (OpamPackage.Name.to_string name)
   | Some opam ->
     let opam =
       match OpamFile.OPAM.get_url opam with
@@ -764,10 +759,26 @@ let list st ~short =
                   (OpamPackage.version_to_string inst))]
         with Not_found -> OpamConsole.colorise `yellow "(uninstalled)", []
       in
+      let vcs_revision =
+        let open OpamStd.Option.Op in
+        url >>| OpamFile.URL.url >>= fun url ->
+        match url.OpamUrl.backend with
+        | #OpamUrl.version_control ->
+          let srcdir = OpamSwitchState.source_dir st nv in
+          let prefix, color, rev =
+            match OpamProcess.Job.run (OpamRepository.revision srcdir url) with
+            | None | exception _ ->
+              ("", `red, "error while fetching current revision")
+            | Some ver -> ("at ", `magenta, ver)
+          in
+          Some (Printf.sprintf "(%s%s)" prefix (OpamConsole.colorise color rev))
+        | _ -> None
+      in
       [ OpamPackage.to_string nv;
         state;
         OpamConsole.colorise `blue kind;
-        String.concat " " (target::extra) ]
+        String.concat " " (target::extra);
+        OpamStd.Option.default "" vcs_revision ]
     with Not_found ->
       [ OpamPackage.to_string nv;
         OpamConsole.colorise `red " (no definition found)" ]
