@@ -867,18 +867,16 @@ let parallel_apply t
 
   let cleanup_artefacts graph =
     PackageActionGraph.iter_vertex (function
-        | `Remove nv
-          when not (OpamPackage.has_name t.pinned nv.name) ->
+        | `Remove nv ->
           OpamAction.cleanup_package_artefacts t nv
           (* if reinstalled, only removes build dir *)
-        | `Install nv
-          when not (OpamPackage.has_name t.pinned nv.name)
-            || OpamSwitchState.is_version_pinned t nv.name ->
-          let build_dir =
-            OpamPath.Switch.build t.switch_global.root t.switch nv in
+        | `Install nv ->
           if not OpamClientConfig.(!r.keep_build_dir) then
+            let build_dir =
+              OpamPath.Switch.build t.switch_global.root t.switch nv
+            in
             OpamFilename.rmdir build_dir
-        | `Remove _ | `Install _ | `Build _ | `Fetch _ -> ()
+        | `Build _ | `Fetch _ -> ()
         | `Change _ | `Reinstall _  -> assert false)
       graph
   in
@@ -1131,58 +1129,77 @@ let syspkgs_to_string spkgs =
     (fun p -> OpamConsole.colorise `bold (OpamSysPkg.to_string p))
     (OpamSysPkg.Set.elements spkgs)
 
-let print_depext_msg (avail, nf) =
-  if not (OpamSysPkg.Set.is_empty nf) then
+let print_depext_msg (status : OpamSysPkg.status) =
+  if not (OpamSysPkg.Set.is_empty status.s_not_found) then
     OpamConsole.warning
       "These additional system packages are required, but not available on \
        your system: %s"
-      (syspkgs_to_string nf);
-  if not (OpamSysPkg.Set.is_empty avail) then
+      (syspkgs_to_string status.s_not_found);
+  if not (OpamSysPkg.Set.is_empty status.s_available) then
     (OpamConsole.formatted_msg
        "\nThe following system packages will first need to be installed:\n";
      OpamConsole.formatted_msg ~indent:4 "    %s\n"
-       (syspkgs_to_string avail))
+       (syspkgs_to_string status.s_available))
 
 (* Gets depexts from the state, without checking again, unless [recover] is
    true. *)
-let get_depexts ?(force=false) ?(recover=false) t packages =
-  if not force && OpamStateConfig.(!r.no_depexts) then OpamSysPkg.Set.empty else
-  let sys_packages =
-    if recover then
-      OpamSwitchState.depexts_status_of_packages t packages
-    else
-      let base = Lazy.force t.sys_packages in
-      (* workaround: st.sys_packages is not always updated with added
-         packages *)
-      let more_pkgs =
-        OpamPackage.Set.filter (fun nv ->
-            (* dirty heuristic: recompute for all non-canonical packages *)
-            OpamPackage.Map.find_opt nv t.repos_package_index
-            <> OpamSwitchState.opam_opt t nv)
-          packages
-      in
-      if OpamPackage.Set.is_empty more_pkgs then base else
-      OpamPackage.Map.union (fun _ x -> x) base
-        (OpamSwitchState.depexts_status_of_packages t more_pkgs)
-  in
-  let avail, nf =
-    OpamPackage.Set.fold (fun pkg (avail,nf) ->
-        match OpamPackage.Map.find_opt pkg sys_packages with
-        | Some sys ->
-          OpamSysPkg.(Set.union avail sys.s_available),
-          OpamSysPkg.(Set.union nf sys.s_not_found)
-        | None -> avail, nf)
-      packages (OpamSysPkg.Set.empty, OpamSysPkg.Set.empty)
-  in
-  print_depext_msg (avail, nf);
-  avail
+let get_depexts ?(force=false) ?(recover=false) t
+    ~pkg_to_install ~pkg_installed =
+  if not force && OpamStateConfig.(!r.no_depexts) then
+    OpamSysPkg.to_install_empty
+  else
+    let sys_packages =
+      if recover then
+        OpamSwitchState.depexts_status_of_packages t pkg_to_install
+      else
+        let base = Lazy.force t.sys_packages in
+        (* workaround: st.sys_packages is not always updated with added
+           packages *)
+        let more_pkgs =
+          OpamPackage.Set.filter (fun nv ->
+              (* dirty heuristic: recompute for all non-canonical packages *)
+              OpamPackage.Map.find_opt nv t.repos_package_index
+              <> OpamSwitchState.opam_opt t nv)
+            pkg_to_install
+        in
+        if OpamPackage.Set.is_empty more_pkgs then base else
+          OpamPackage.Map.union (fun _ x -> x) base
+            (OpamSwitchState.depexts_status_of_packages t more_pkgs)
+    in
+    let already_installed = OpamPackage.Set.diff pkg_installed pkg_to_install in
+    let open OpamSysPkg.Set.Op in
+    let status =
+      OpamPackage.Set.fold (fun pkg (acc : OpamSysPkg.status) ->
+          match OpamPackage.Map.find_opt pkg sys_packages with
+          | Some sys ->
+            { OpamSysPkg.
+              s_available = acc.s_available ++ sys.s_available;
+              s_not_found = acc.s_not_found ++ sys.s_not_found;
+            }
+          | None -> acc)
+        pkg_to_install OpamSysPkg.status_empty
+    in
+    print_depext_msg status;
+    let ti_required =
+      let bypass = t.switch_config.OpamFile.Switch_config.depext_bypass in
+      OpamPackage.Set.fold (fun nv req ->
+          OpamSwitchState.depexts t nv -- bypass ++ req)
+        already_installed OpamSysPkg.Set.empty
+    in
+    { ti_new = status.s_available; ti_required }
 
-let install_sys_packages ~map_sysmap ~confirm env config sys_packages t =
+(* propagate_st is used to determine if we need to handle stateless depext
+   systems *)
+let install_sys_packages_t ~propagate_st ~map_sysmap ~confirm env config
+    sys_packages t =
   let rec entry_point t sys_packages =
     if OpamClientConfig.(!r.fake) then
       (print_command sys_packages; t)
     else if OpamFile.Config.depext_run_installs config then
-      if confirm then menu t sys_packages else auto_install t sys_packages
+      if confirm then
+        menu t sys_packages
+      else
+        auto_install t sys_packages
     else
       manual_install t sys_packages
   and menu t sys_packages =
@@ -1215,17 +1232,13 @@ let install_sys_packages ~map_sysmap ~confirm env config sys_packages t =
       OpamConsole.msg "\n";
       print_command sys_packages;
       OpamConsole.pause "Standing by, press enter to continue when done.";
-      OpamConsole.msg "\n";
       check_again t sys_packages
     | `Ignore -> bypass t
     | `Quit -> give_up_msg (); OpamStd.Sys.exit_because `Aborted
   and print_command sys_packages =
-    (* Ensure that setup-x86_64.exe exists, so that an invalid command is not
-       displayed to the user. *)
-    if OpamSysPoll.os_distribution env = Some "cygwin" then
-      OpamSysInteract.Cygwin.check_setup ~update:false;
     let commands =
-      OpamSysInteract.install_packages_commands ~env config sys_packages
+      OpamSysInteract.install_packages_commands ~env
+        (propagate_st t) config sys_packages
       |> List.map (fun ((`AsAdmin c | `AsUser c), a) -> c::a)
     in
     OpamConsole.formatted_msg
@@ -1254,33 +1267,33 @@ let install_sys_packages ~map_sysmap ~confirm env config sys_packages t =
     | `Quit -> give_up ()
   and auto_install t sys_packages =
     try
-      if OpamSysPoll.os_distribution env = Some "cygwin" then
-        OpamSysInteract.Cygwin.check_setup ~update:true;
-      OpamSysInteract.install ~env config sys_packages; (* handles dry_run *)
+      OpamSysInteract.install ~env
+        (propagate_st t) config sys_packages; (* handles dry_run *)
       map_sysmap (fun _ -> OpamSysPkg.Set.empty) t
     with Failure msg ->
       OpamConsole.error "%s" msg;
       check_again t sys_packages
   and check_again t sys_packages =
     let open OpamSysPkg.Set.Op in
-    let needed, notfound =
-      OpamSysInteract.packages_status ~env config sys_packages
+    let status =
+      OpamSysInteract.packages_status ~env config sys_packages.ti_new
     in
-    let still_missing = needed ++ notfound in
-    let installed = sys_packages -- still_missing in
+    let still_missing = status.s_available ++ status.s_not_found in
+    let installed = sys_packages.ti_new -- still_missing in
     let t =
       map_sysmap (fun sysp -> OpamSysPkg.Set.diff sysp installed) t
     in
     if OpamSysPkg.Set.is_empty still_missing then t else
-    if OpamSysPkg.Set.(not (is_empty notfound) && is_empty needed) then
+    if not (OpamSysPkg.Set.is_empty status.s_not_found)
+    && OpamSysPkg.Set.is_empty status.s_available then
       (OpamConsole.error
          "These packages are still missing and not found via \
           your system package manager: %s\n"
-         (syspkgs_to_string notfound);
-       manual_install t still_missing)
+         (syspkgs_to_string status.s_not_found);
+       manual_install t { sys_packages with ti_new = still_missing })
     else
       (OpamConsole.error "These packages are still missing: %s\n"
-         (syspkgs_to_string sys_packages);
+         (syspkgs_to_string sys_packages.ti_new);
        if OpamStd.Sys.tty_in then entry_point t sys_packages
        else give_up ())
   and bypass t =
@@ -1300,7 +1313,7 @@ let install_sys_packages ~map_sysmap ~confirm env config sys_packages t =
     give_up_msg ();
     OpamStd.Sys.exit_because `Aborted
   in
-  if OpamSysPkg.Set.is_empty sys_packages ||
+  if (OpamSysPkg.Set.is_empty sys_packages.OpamSysPkg.ti_new) ||
      OpamClientConfig.(!r.show) ||
      OpamClientConfig.(!r.assume_depexts) then
     t
@@ -1311,7 +1324,8 @@ let install_sys_packages ~map_sysmap ~confirm env config sys_packages t =
     entry_point t sys_packages
   with Sys.Break as e -> OpamStd.Exn.finalise e give_up_msg
 
-let install_depexts ?(force_depext=false) ?(confirm=true) t packages =
+let install_depexts ?(force_depext=false) ?(confirm=true) t
+    ~pkg_to_install ~pkg_installed =
   let map_sysmap f t =
     let sys_packages =
       OpamPackage.Set.fold (fun nv sys_map ->
@@ -1322,7 +1336,7 @@ let install_depexts ?(force_depext=false) ?(confirm=true) t packages =
                                  f status.OpamSysPkg.s_available }
               sys_map
           | None -> sys_map)
-        packages
+        pkg_to_install
         (Lazy.force t.sys_packages)
     in
     { t with sys_packages = lazy sys_packages }
@@ -1331,14 +1345,17 @@ let install_depexts ?(force_depext=false) ?(confirm=true) t packages =
     confirm && not (OpamSysInteract.Cygwin.is_internal t.switch_global.config)
   in
   let sys_packages =
-    get_depexts ~force:force_depext ~recover:force_depext t packages
+    get_depexts ~force:force_depext ~recover:force_depext t
+      ~pkg_to_install ~pkg_installed
   in
   let env = t.switch_global.global_variables in
   let config = t.switch_global.config in
-  install_sys_packages ~map_sysmap ~confirm env config sys_packages t
+  install_sys_packages_t ~propagate_st:OpamStd.Option.some ~map_sysmap ~confirm
+    env config sys_packages t
 
 let install_sys_packages ~confirm =
-  install_sys_packages ~map_sysmap:(fun _ () -> ()) ~confirm
+  install_sys_packages_t ~propagate_st:(fun () -> None)
+    ~map_sysmap:(fun _ () -> ()) ~confirm
 
 (* Apply a solution *)
 let apply ?ask t ~requested ?print_requested ?add_roots
@@ -1361,9 +1378,13 @@ let apply ?ask t ~requested ?print_requested ?add_roots
     in
     let t =
       if OpamClientConfig.(!r.show) then
-        let _ = get_depexts t virt_inst in t
+        let _ : OpamSysPkg.to_install =
+          get_depexts t ~pkg_to_install:virt_inst ~pkg_installed:t.installed
+        in
+        t
         (* Prints the msg about additional depexts to install *)
-      else install_depexts t virt_inst
+      else
+        install_depexts t ~pkg_to_install:virt_inst ~pkg_installed:t.installed
     in
     t, Nothing_to_do
   else (
@@ -1417,13 +1438,19 @@ let apply ?ask t ~requested ?print_requested ?add_roots
         solution0;
     );
     if OpamClientConfig.(!r.show) then
-      let _ = get_depexts t new_state0.installed in
+      let _ : OpamSysPkg.to_install =
+        get_depexts t
+          ~pkg_to_install:new_state0.installed
+          ~pkg_installed:new_state0.installed
+      in
       (* Prints the msg about additional depexts to install *)
-      t, Aborted
+      (t, Aborted)
     else if download_only || confirmation ?ask names solution then (
       let t =
-        install_depexts t  @@ OpamPackage.Set.inter
-          new_state0.installed (OpamSolver.all_packages solution0)
+        install_depexts t
+          ~pkg_to_install:(OpamPackage.Set.inter new_state0.installed
+                             (OpamSolver.all_packages solution0))
+          ~pkg_installed:new_state0.installed
       in
       let requested =
         OpamPackage.packages_of_names new_state.installed names
