@@ -72,13 +72,8 @@ let with_tmp_dir_job fjob =
 let rmdir dirname =
   OpamSystem.remove_dir (Dir.to_string dirname)
 
-let rec rmdir_cleanup dirname =
-  let sd = Dir.to_string dirname in
-  if OpamSystem.dir_is_empty sd then (
-    rmdir dirname;
-    let parent = Filename.dirname sd in
-    if parent <> sd then rmdir_cleanup parent
-  )
+let rmdir_cleanup dirname =
+  OpamSystem.rmdir_cleanup (Dir.to_string dirname)
 
 let cwd () =
   Dir.of_string (Unix.getcwd ())
@@ -266,6 +261,12 @@ let exists filename =
 let opt_file filename =
   if exists filename then Some filename else None
 
+let with_tmp_file fn =
+  OpamSystem.with_tmp_file (fun file -> fn (of_string file))
+
+let with_tmp_file_job fjob =
+  OpamSystem.with_tmp_file_job (fun file -> fjob (of_string file))
+
 let with_contents fn filename =
   fn (read filename)
 
@@ -293,8 +294,11 @@ let files_and_links d =
 let copy ~src ~dst =
   if src <> dst then OpamSystem.copy_file (to_string src) (to_string dst)
 
-let copy_dir ~src ~dst =
-  if src <> dst then OpamSystem.copy_dir (Dir.to_string src) (Dir.to_string dst)
+let copy_dir_t f ~src ~dst =
+  if src <> dst then f (Dir.to_string src) (Dir.to_string dst)
+
+let copy_dir = copy_dir_t OpamSystem.copy_dir
+let copy_dir_except_vcs = copy_dir_t OpamSystem.copy_dir_except_vcs
 
 let install ?warning ?exec ~src ~dst () =
   if src <> dst then OpamSystem.install ?warning ?exec (to_string src) (to_string dst)
@@ -439,8 +443,9 @@ let link ?(relative=false) ~target ~link =
   OpamSystem.link target (to_string link)
 [@@ocaml.warning "-16"]
 
-let patch ?preprocess filename dirname =
-  OpamSystem.patch ?preprocess ~dir:(Dir.to_string dirname) (to_string filename)
+let patch ?preprocess ~allow_unclean filename dirname =
+  OpamSystem.patch ?preprocess ~allow_unclean ~dir:(Dir.to_string dirname)
+    (to_string filename)
 
 let flock flag ?dontblock file = OpamSystem.flock flag ?dontblock (to_string file)
 
