@@ -50,7 +50,9 @@ val mkdir: Dir.t -> unit
 (** List the sub-directory recursively *)
 val rec_dirs: Dir.t -> Dir.t list
 
-val dir_is_empty: Dir.t -> bool
+(** Returns whether a directory is empty.
+    Returns [None] if the directory could not be found. *)
+val dir_is_empty: Dir.t -> bool option
 
 (** List the sub-directory (do not recurse) *)
 val dirs: Dir.t -> Dir.t list
@@ -72,6 +74,10 @@ val move_dir: src:Dir.t -> dst:Dir.t -> unit
     [src] into [dst], overwriting any existing files. *)
 val copy_dir: src:Dir.t -> dst:Dir.t -> unit
 
+(** Same as [copy_dir] except it avoids copying VCS directories
+    ([.git], [.hg], [_darcs]) *)
+val copy_dir_except_vcs : src:Dir.t -> dst:Dir.t -> unit
+
 (** Link a directory *)
 val link_dir: target:Dir.t -> link:Dir.t -> unit
 
@@ -170,6 +176,13 @@ val exists: t -> bool
     a symlink to one *)
 val opt_file: t -> t option
 
+(** Execute a function with a file in a temp directory.
+    It is always cleaned up afterwards. *)
+val with_tmp_file: (t -> 'a) -> 'a
+
+(** Provide an automatically cleaned up file in temp directory to a job *)
+val with_tmp_file_job: (t -> 'a OpamProcess.job) -> 'a OpamProcess.job
+
 (** Check whether a file has a given suffix *)
 val check_suffix: t -> string -> bool
 
@@ -264,8 +277,12 @@ val remove_prefix_dir: Dir.t -> Dir.t -> string
 val remove_suffix: Base.t -> t -> string
 
 (** Apply a patch in a directory. If [preprocess] is set to false, there is no
-    CRLF translation. Returns [None] on success, the process error otherwise *)
-val patch: ?preprocess:bool -> t -> Dir.t -> exn option OpamProcess.job
+    CRLF translation. Returns [None] on success, the process error otherwise.
+
+    @param allow_unclean decides if applying a patch on a directory which
+    differs slightly from the one described in the patch file is allowed.
+    Allowing unclean applications imitates the default behaviour of GNU Patch. *)
+val patch: ?preprocess:bool -> allow_unclean:bool -> t -> Dir.t -> exn option
 
 (** Create an empty file *)
 val touch: t -> unit
@@ -282,20 +299,22 @@ val find_in_parents: (Dir.t -> bool) ->  Dir.t -> Dir.t option
 
 (** {2 Locking} *)
 
-(** See [OpamSystem.flock]. Prefer the higher level [with_flock] functions when
+(** See {!OpamSystem.flock}.  Prefer the higher level [with_flock] functions when
     possible *)
 val flock: [< OpamSystem.lock_flag ] -> ?dontblock:bool -> t -> OpamSystem.lock
 
 (** Calls [f] while holding a lock file. Ensures the lock is properly released
-    on [f] exit. Raises [OpamSystem.Locked] if [dontblock] is set and the lock
-    can't be acquired. [f] is passed the file_descr of the lock. *)
+    on [f] exit. [f] is passed the file_descr of the lock.
+    @raise OpamSystem.Locked if [dontblock] is set and the lock
+    can't be acquired. *)
 val with_flock: [< OpamSystem.lock_flag ] -> ?dontblock:bool -> t ->
   (Unix.file_descr -> 'a) -> 'a
 
 (** Calls [f] with the file lock upgraded to at least [flag], then restores the
     previous lock level. Upgrade to [`Lock_write] should never be used in
-    blocking mode as it would deadlock. Raises [OpamSystem.Locked] (but keeps
-    the lock as is) if [dontblock] is set and the lock can't be upgraded. *)
+    blocking mode as it would deadlock.
+    @raise OpamSystem.Locked (but keeps the lock as is) if [dontblock] is set
+    and the lock can't be upgraded. *)
 val with_flock_upgrade:
   [< OpamSystem.actual_lock_flag ] -> ?dontblock:bool -> OpamSystem.lock -> (Unix.file_descr -> 'a) -> 'a
 
