@@ -15,6 +15,9 @@ type t
 
 val kind: t -> kind
 
+(** The list of all the possible values of kind *)
+val all_kinds : kind list
+
 (** The value of the hash, as a string of hexadecimal characters *)
 val contents: t -> string
 
@@ -41,7 +44,7 @@ val sort : t list -> t list
 
 val check_file: string -> t -> bool
 
-(** Like [check_file], but returns the actual mismatching hash of the file, or
+(** Like {!check_file}, but returns the actual mismatching hash of the file, or
     [None] in case of match *)
 val mismatch: string -> t -> t option
 
