@@ -22,8 +22,9 @@ module type SET = sig
 
   val is_singleton: t -> bool
 
-  (** Returns one element, assuming the set is a singleton. Raises [Not_found]
-      on an empty set, [Failure] on a non-singleton. *)
+  (** Returns one element, assuming the set is a singleton.
+      @raise Not_found on an empty set
+      @raise Failure on a non-singleton *)
   val choose_one : t -> elt
 
   val choose_opt: t -> elt option
@@ -36,7 +37,7 @@ module type SET = sig
   val find: (elt -> bool) -> t -> elt
   val find_opt: (elt -> bool) -> t -> elt option
 
-  (** Raises Failure in case the element is already present *)
+  (** @raise Failure in case the element is already present *)
   val safe_add: elt -> t -> t
 
   (** Accumulates the resulting sets of a function of elements until a fixpoint
@@ -44,8 +45,8 @@ module type SET = sig
   val fixpoint: (elt -> t) -> t -> t
 
   (** [map_reduce f op t] applies [f] to every element of [t] and combines the
-      results using associative operator [op]. Raises [Invalid_argument] on an
-      empty set, or returns [default] if it is defined. *)
+      results using associative operator [op].
+      @raise Invalid_argument on an empty set if [default] is not defined *)
   val map_reduce: ?default:'a -> (elt -> 'a) -> ('a -> 'a -> 'a) -> t -> 'a
 
   module Op : sig
@@ -81,7 +82,7 @@ module type MAP = sig
 
   val of_list: (key * 'a) list -> 'a t
 
-  (** Raises Failure in case the element is already present *)
+  (** @raise Failure in case the element is already present *)
   val safe_add: key -> 'a -> 'a t -> 'a t
 
   (** [update k f zero map] updates the binding of [k] in [map] using function
@@ -89,8 +90,8 @@ module type MAP = sig
   val update: key -> ('a -> 'a) -> 'a -> 'a t -> 'a t
 
   (** [map_reduce f op t] applies [f] to every binding of [t] and combines the
-      results using associative operator [op]. Raises [Invalid_argument] on an
-      empty map, or returns [default] if it is defined. *)
+      results using associative operator [op].
+      @raise Invalid_argument on an empty map if [default] is not defined *)
   val map_reduce:
     ?default:'b -> (key -> 'a -> 'b) -> ('b -> 'b -> 'b) -> 'a t -> 'b
 
@@ -162,6 +163,8 @@ module Option: sig
 
   val equal: ('a -> 'a -> bool) -> 'a option -> 'a option -> bool
 
+  val equal_some : ('a -> 'a -> bool) -> 'a -> 'a option -> bool
+
   val to_string: ?none:string -> ('a -> string) -> 'a option -> string
 
   val to_list: 'a option -> 'a list
@@ -186,13 +189,13 @@ module List : sig
 
   val cons: 'a -> 'a list -> 'a list
 
-  (** Convert list items to string and concat. [sconcat_map sep f x] is equivalent
-      to String.concat sep (List.map f x) but tail-rec. *)
+  (** Convert list items to string and concat. [concat_map sep f x] is equivalent
+      to [String.concat sep (List.map f x)] but tail-rec. *)
   val concat_map:
     ?left:string -> ?right:string -> ?nil:string -> ?last_sep:string ->
     string -> ('a -> string) -> 'a list -> string
 
-  (** Like [List.find], but returning option instead of raising *)
+  (** Like {!Stdlib.List.find}, but returning option instead of raising *)
   val find_opt: ('a -> bool) -> 'a list -> 'a option
 
   val to_string: ('a -> string) -> 'a list -> string
@@ -214,7 +217,8 @@ module List : sig
       @raise Not_found if all of them yield [None] *)
   val find_map: ('a -> 'b option) -> 'a list -> 'b
 
-  (** Like [find_map], but returns [Some _] if succeeded and [None] if failed. *)
+  (** Like {!find_map},
+  but returns [Some _] if succeeded and [None] if failed. *)
   val find_map_opt: ('a -> 'b option) -> 'a list -> 'b option
 
   (** Insert a value in an ordered list *)
@@ -224,19 +228,19 @@ module List : sig
       end if index < 0 or > length respectively). Not tail-recursive *)
   val insert_at: int -> 'a -> 'a list -> 'a list
 
-  (** Like [List.assoc] with an equality function. *)
+  (** Like {!List.assoc} with an equality function. *)
   val assoc: ('a -> 'a -> bool) -> 'a -> ('a * 'b) list -> 'b
 
-  (** Like [assoc], but returning option instead of raising [Not_found] *)
+  (** Like {!assoc}, but returning option instead of raising [Not_found] *)
   val assoc_opt: ('a -> 'a -> bool) -> 'a -> ('a * 'b) list -> 'b option
 
-  (** Like [assoc], but as an option, and also returns the list with the
+  (** Like {!assoc}, but as an option, and also returns the list with the
       binding removed, e.g. equivalent to [(assoc_opt x l, remove_assoc x l)]
       (but tail-recursive and more efficient) *)
   val pick_assoc:
     ('a -> 'a -> bool) -> 'a -> ('a * 'b) list -> 'b option * ('a * 'b) list
 
-  (** Like [assoc], but returns a boolean instead of associated value *)
+  (** Like {!assoc}, but returns a boolean instead of associated value *)
   val mem_assoc: ('a -> 'a -> bool) -> 'a -> ('a * 'b) list -> bool
 
   (** [remove_assoc eq k l] removes first association of [k] from list [l]
@@ -250,10 +254,11 @@ module List : sig
   val update_assoc:
     ('a -> 'a -> bool) -> 'a -> 'b -> ('a * 'b) list -> ('a * 'b) list
 
-  (** Like [pick_assoc], but with a test function that takes a list element *)
+  (** Like {!pick_assoc}, but with a test function that takes a list element *)
   val pick: ('a -> bool) -> 'a list -> 'a option * 'a list
 
-  (** Like [List.fold_left], but also performs [List.map] at the same time *)
+  (** Like {!Stdlib.List.fold_left}, but also performs {!Stdlib.List.map} at
+      the same time *)
   val fold_left_map: ('s -> 'a -> ('s * 'b)) -> 's -> 'a list -> 's * 'b list
 end
 
@@ -281,8 +286,8 @@ module String : sig
   val exact_match: Re.re -> string -> bool
   val find_from: (char -> bool) -> string -> int -> int
 
-  (** Like [String.compare], but with lowercase/uppercase variants ordered next
-      to each other (still considered not equal though) *)
+  (** Like {!Stdlib.String.compare}, but with lowercase/uppercase variants
+      ordered next to each other (still considered not equal though) *)
   val compare_case: string -> string -> int
 
   (** {3 Manipulation} *)
@@ -303,14 +308,14 @@ module String : sig
   (** Cut a string at the first occurrence of the given char *)
   val cut_at: string -> char -> (string * string) option
 
-  (** Same as [cut_at], but starts from the right *)
+  (** Same as {!cut_at}, but starts from the right *)
   val rcut_at: string -> char -> (string * string) option
 
   (** Split a string at occurrences of a given characters. Empty strings are
       skipped. *)
   val split: string -> char -> string list
 
-  (** The same as [split], but keep empty strings (leading, trailing or between
+  (** The same as {!split}, but keep empty strings (leading, trailing or between
       contiguous delimiters) *)
   val split_delim: string -> char -> string list
 
@@ -371,7 +376,7 @@ module Exn : sig
   (** To use when catching default exceptions: ensures we don't catch fatal errors
       like C-c. try-with should _always_ (by decreasing order of preference):
       - either catch specific exceptions
-      - or re-raise the same exception (preferably with [Exn.finalise])
+      - or re-raise the same exception (preferably with {!Exn.finalise})
       - or call this function on the caught exception *)
   val fatal: exn -> unit
 
@@ -388,7 +393,7 @@ module Exn : sig
   val finalise: exn -> (unit -> unit) -> 'a
 
   (** Execute the given continuation, then run the finaliser before returning
-      the result. If an exception is raised, call [finalise] with the given
+      the result. If an exception is raised, call {!finalise} with the given
       finaliser. *)
   val finally: (unit -> unit) -> (unit -> 'a) -> 'a
 
@@ -510,11 +515,15 @@ module Sys : sig
   (** Queried lazily *)
   val os: unit -> os
 
-  (** The output of the command "uname", with the given argument. Memoised. *)
-  val uname: string -> string option
+  (** The output of the command "uname -U". FreeBSD only. Reasoning:
+      https://github.com/ocaml/opam/pull/4274#issuecomment-659280485 *)
+  val get_freebsd_version: unit -> string option
 
-  (** The output of the command "getconf", with the given argument. Memoised. *)
-  val getconf: string -> string option
+  (** The output of the command "getconf LONG_BIT". *)
+  val get_long_bit: unit -> string option
+
+  (** The memoized result of the uname function from the C standard library *)
+  val uname : unit -> OpamStubs.uname
 
   (** Append .exe (only if missing) to executable filenames on Windows *)
   val executable_name : string -> string
@@ -609,7 +618,7 @@ module Sys : sig
       [Unix.execvpe]. *)
   exception Exec of string * string array * string array
 
-  (** Raises [Exit i] *)
+  (** Raise exception {!Exit} [i] *)
   (* val exit: int -> 'a *)
 
   type exit_reason =
@@ -622,7 +631,7 @@ module Sys : sig
 
   val get_exit_code : exit_reason -> int
 
-  (** Raises [Exit], with the code associated to the exit reason *)
+  (** Raise exception {!Exit}, with the code associated to the exit reason *)
   val exit_because: exit_reason -> 'a
 
   (**/**)
@@ -642,7 +651,7 @@ module Win32 : sig
 
   val set_parent_pid : int32 -> unit
   (** Change which the pid written to by {!parent_putenv}. This function cannot
-      be called after [parent_putenv]. *)
+      be called after {!parent_putenv}. *)
 
   val parent_putenv : string -> string -> bool
   (** Update an environment variable in the parent (i.e. shell) process's
@@ -693,7 +702,7 @@ module Config : sig
   val env_int: env_var -> int option
 
   type level = int
-  (* Like [env_int], but accept boolean values for 0 and 1 *)
+  (* Like {!env_int}, but accept boolean values for 0 and 1 *)
   val env_level: env_var -> level option
 
   type sections = int option String.Map.t
