@@ -41,7 +41,8 @@ module type IO_FILE = sig
   (** Write some contents to a file *)
   val write: t typed_file -> t -> unit
 
-  (** Read file contents. Raise an error if the file does not exist. *)
+  (** Read file contents.
+      @raise OpamSystem.Internal_error if the file does not exist. *)
   val read: t typed_file -> t
 
   (** Returns [None] on non-existing file *)
@@ -60,7 +61,7 @@ module type IO_FILE = sig
 
 end
 
-(* Error less [IO_FILE] read functions. *)
+(* Error less {!IO_FILE} read functions. *)
 module type BestEffortRead = sig
   type t
   val read: t typed_file -> t
@@ -1028,7 +1029,7 @@ module Repo_config_legacy : sig
 end
 
 module Repos_config: sig
-  type t = (url * trust_anchors option) option OpamRepositoryName.Map.t
+  type t = (url * trust_anchors option) OpamRepositoryName.Map.t
   include IO_FILE with type t := t
   module BestEffort: BestEffortRead with type t := t
 end
@@ -1128,13 +1129,6 @@ end
 (** {2 urls.txt file *} *)
 module File_attributes: IO_FILE with type t = file_attribute_set
 
-module Stats: sig
-
-  (** Display statistics about file access. *)
-  val print: unit -> unit
-
-end
-
 (** Helper module for manipulation of the raw syntax ([opamfile]) format.
     (the specific file handling modules are derived from this one) *)
 module Syntax : sig
