@@ -180,9 +180,12 @@ module Switch: sig
   (** Cached environment updates. *)
   val environment: t -> switch -> OpamFile.Environment.t OpamFile.t
 
-  (** Like [environment], but from the switch prefix dir *)
+  (** Like {!environment}, but from the switch prefix dir *)
   val env_relative_to_prefix: dirname -> OpamFile.Environment.t OpamFile.t
 
+  (** A Nix specific environment file *)
+  val nix_env : t -> switch -> OpamFile.Environment.t OpamFile.t
+
   (** Directory where the metadata of installed packages is mirrored.
       {i $meta/packages/} *)
   val installed_opams: t -> switch -> dirname
