@@ -332,3 +332,16 @@ let string_of_path_format = function
 let char_of_separator = function
   | SSemiColon -> ';'
   | SColon -> ':'
+
+let switch_selections_compare x
+    {sel_installed; sel_roots; sel_compiler; sel_pinned} =
+  let cmp = OpamPackage.Set.compare x.sel_installed sel_installed in
+  if cmp <> 0 then cmp else
+    let cmp = OpamPackage.Set.compare x.sel_roots sel_roots in
+    if cmp <> 0 then cmp else
+      let cmp = OpamPackage.Set.compare x.sel_compiler sel_compiler in
+      if cmp <> 0 then cmp else
+        OpamPackage.Set.compare x.sel_pinned sel_pinned
+
+let switch_selections_equal x y =
+  switch_selections_compare x y = 0
