@@ -53,16 +53,17 @@ type 'a options_fun =
   'a
 
 let default = {
+  (* Keep synchronised with [OpamInitDefaults.req_dl_tools] *)
   download_tool = lazy (
     let os = OpamStd.Sys.os () in
     try
       let curl = "curl", `Curl in
       let tools =
         match os with
-        | Darwin  -> ["wget", `Default; curl]
-        | FreeBSD -> ["fetch", `Default ; curl]
-        | OpenBSD -> ["ftp", `Default; curl]
-        | _ -> [curl; "wget", `Default]
+        | FreeBSD | DragonFly -> [curl; "fetch", `Default]
+        | OpenBSD | NetBSD -> [curl; "ftp", `Default]
+        | Linux | Darwin | Cygwin | Win32
+        | Unix | Other _ -> [curl; "wget", `Default]
       in
       let cmd, kind =
         List.find (fun (c,_) -> OpamSystem.resolve_command c <> None) tools
