@@ -677,11 +677,22 @@ let updates_common ~set_opamroot ~set_opamswitch root switch =
     else [] in
   root @ switch
 
+let updates_nix st =
+  match OpamSysPoll.os_family st.switch_global.global_variables with
+  | Some "nixos" ->
+    (match
+       OpamFile.Environment.read_opt
+         (OpamPath.Switch.nix_env st.switch_global.root st.switch)
+     with
+     | None -> []
+     | Some env -> List.map resolve_separator_and_format env)
+  | _ -> []
+
 let updates ~set_opamroot ~set_opamswitch ?force_path st =
   let common =
     updates_common ~set_opamroot ~set_opamswitch st.switch_global.root st.switch
   in
-  common @ compute_updates ?force_path st
+  common @ compute_updates ?force_path st @ updates_nix st
 
 let get_pure ?(updates=[]) () =
   let env = List.map (fun (v,va) -> v,va,None) (OpamStd.Env.list ()) in
@@ -1184,7 +1195,20 @@ let write_custom_init_scripts root custom =
     ) custom
 
 let write_dynamic_init_scripts st =
-  let updates = updates ~set_opamroot:false ~set_opamswitch:false st in
+  (* Empty environment updates must not be written to the scripts *)
+  let is_not_empty_update
+    : (_, OpamTypes.euok_writeable) OpamTypes.env_update -> _ = function
+  | {envu_value = ""; envu_op; _} ->
+    begin match envu_op with
+    | PlusEq | EqPlus | ColonEq | EqColon | EqPlusEq -> false
+    | Eq -> true
+    end
+  | _ -> true
+  in
+  let updates =
+    updates ~set_opamroot:false ~set_opamswitch:false st
+    |> List.filter is_not_empty_update
+  in
   try
     if OpamStateConfig.is_newer_than_self
         ~lock_kind:`Lock_write st.switch_global then
